import React, { useEffect, useContext, useState } from 'react'
import { AppContext } from 'components/Contexts/AppContextProvider'
import {
  TimeStep,
  IPerformanceIndicator,
  ITimePeriod,
} from 'services/dataConsumptionContracts'
import { FluidType } from 'enum/fluid.enum'
import { withClient, Client } from 'cozy-client'
import PerformanceIndicator from 'components/ContentComponents/PerformanceIndicator/PerformanceIndicator'
import ConsumptionPeriodSelector from 'services/consumptionPeriodSelectorService'
import ConsumptionDataManager from 'services/consumptionDataManagerService'
import PerformanceIndicatorAggregateCalculator from 'services/performanceIndicatorAggregateCalculatorService'
import { translate } from 'cozy-ui/react/I18n'
import { convertDateByTimeStep } from 'utils/utils'
import RedirectionMiniCard from 'components/ContentComponents/Card/RedirectionMiniCard'

interface SingleFluidIndicatorsContainerProps {
  timeStep: TimeStep
  setIndicatorsLoaded(): void
  client: Client
  fluidTypes: FluidType[]
  t: Function
}

const SingleFluidIndicatorsContainer: React.FC<SingleFluidIndicatorsContainerProps> = ({
  timeStep,
  setIndicatorsLoaded,
  client,
  fluidTypes,
  t,
}: SingleFluidIndicatorsContainerProps) => {
  const appContext = useContext(AppContext)
  const allConfiguredFluidTypes = appContext.fluidTypes

  const [performanceIndicators, setPerformanceIndicators] = useState<
    IPerformanceIndicator[]
  >([])
  const [
    currentTimePeriod,
    setCurrentTimePeriod,
  ] = useState<ITimePeriod | null>(null)
  const [
    comparisonTimePeriod,
    setComparisonTimePeriod,
  ] = useState<ITimePeriod | null>(null)
  const filteredFluidTypes = allConfiguredFluidTypes.filter(
    fluidType => fluidType !== fluidTypes[0]
  )
  const [isLoaded, setIsLoaded] = useState<boolean>(false)

  useEffect(() => {
    async function populatePerformanceIndicators() {
      const cdm = new ConsumptionDataManager(client)
      const lastDate = await cdm.fetchLastDateData(fluidTypes)

      if (lastDate) {
        const consumptionPeriodSelector = new ConsumptionPeriodSelector()
        const periods = consumptionPeriodSelector.getTimePeriods(
          lastDate,
          fluidTypes,
          timeStep
        )
        const fetchedPerformanceIndicators = await cdm.getPerformanceIndicators(
          periods.timePeriod,
          timeStep,
          fluidTypes,
          periods.comparisonTimePeriod
        )

        if (fetchedPerformanceIndicators) {
          setPerformanceIndicators(fetchedPerformanceIndicators)
          setCurrentTimePeriod(periods.timePeriod)
          setComparisonTimePeriod(periods.comparisonTimePeriod)
          setIndicatorsLoaded()
          setIsLoaded(true)
        }
      }
    }

    populatePerformanceIndicators()
  }, [timeStep])

  return (
    <>
      {isLoaded ? (
        <div className="fi-root">
          <div className="sfi-content">
            <div className="sfi-header text-14-normal-uppercase">
              {t('COMMON.CONSO_CARDS_LABEL')}{' '}
              {convertDateByTimeStep(currentTimePeriod, timeStep, true)}
            </div>

            <PerformanceIndicator
              performanceIndicator={PerformanceIndicatorAggregateCalculator.aggregatePerformanceIndicators(
                performanceIndicators
              )}
              timePeriodText={convertDateByTimeStep(
                comparisonTimePeriod,
                timeStep
              )}
              timeStep={timeStep}
            />

            {filteredFluidTypes && filteredFluidTypes.length > 0 && (
              <>
                <div className="sfi-header text-14-normal-uppercase">
                  {t('COMMON.MINI_CARDS_LABEL')}
                </div>
                <div className="sfi-redirect">
                  <RedirectionMiniCard fluidTypes={filteredFluidTypes} />
                </div>
              </>
            )}
          </div>
        </div>
      ) : null}
    </>
  )
}

export default translate()(withClient(SingleFluidIndicatorsContainer))
